package logicNode.parser;

import java.util.Scanner;
import java.util.regex.Pattern;

import logicNode.*;

public class LogicNodeParser {

	Scanner scan;
	public Dictionary dict;
	public LogicNodeScope scope;
	private Pattern tokenPat;
	private Pattern namePat;
	
	Token nextToken;
	public Token orToken;
	public Token andToken;
	public Token notToken;
	public Token parenToken;
	public Token closeParenToken;
	private Pattern anyPat;
	
	public LogicNodeParser(String str) {
		scan = new Scanner(str);
		initialize();
	}
	
	public void initReservedWord() {
		orToken = dict.reserve(";");
		andToken = dict.reserve(",");
		parenToken = dict.reserve("(");
		notToken = dict.reserve("~");
		closeParenToken = dict.reserve(")");
	}

	public void initialize() {
		tokenPat = Pattern.compile("[(){},;~]");
		namePat = Pattern.compile("[a-zA-Z][_\\w\\d]*");
		anyPat = Pattern.compile(".");
		dict = new Dictionary();
		scope = new LogicNodeScope(dict);
		scan.useDelimiter("");
		initReservedWord();
	}
	
	public LogicNode parse() {
		nextToken();
		return expr1();
	}
	
	public LogicNode expr1() {
		LogicNode n = expr2();
		while(nextToken == orToken) {
			nextToken();
			n = new OrNode(n, expr2());
		}
		return n;
	}

	public LogicNode expr2() {
		LogicNode n = expr3();
		while(nextToken == andToken) {
			nextToken();
			n = new AndNode(n, expr3());
		}
		return n;
	}
	
	public LogicNode expr3() {
		if (nextToken==notToken) {
			nextToken();
			return new NotNode(term());
		}
		return term();
	}
	
	private LogicNode term() {
		LogicNode n = null;
		if (nextToken==parenToken) {
			nextToken();
			n = expr1();
			if (nextToken==closeParenToken) {
			} else { // syntax error;
			}
		} else if (nextToken == null) {
			return null;
		} else if (nextToken.isVariable()) {
			n =  new VariableNode(nextToken.name(),true);
		}
		nextToken();
		return n;
	}

	public Token nextToken() {
		String s;
		nextToken = null;
		if (scan.hasNext(tokenPat)) {
			s = scan.next(tokenPat);
			nextToken = dict.get(s);
		} else if (scan.hasNext(namePat)) {
			s=scan.next(namePat);
			Token t;
			if ((t = dict.get(s))==null) {
				dict.put(s, t = new Token(s,true));
			}
			nextToken = t;
		} else if (scan.hasNext(anyPat)){
			nextToken = new Token(scan.next(anyPat));
			System.err.println("Can't parse '"+nextToken.name+"'.");
		} else {
			return null;
		}
		return nextToken;
	}
}
